/* domains.c
     COPYRIGHT
          Both this software and its documentation are
 
              Copyrighted 1997 by Vincent Loechner.

          Permission is granted to copy, use, and distribute
          for any commercial or noncommercial purpose under the terms
          of the GNU General Public license, version 2, June 1991
          (see file : LICENSING).
*/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

#include <polylib/polylib32.h>

#include "gtk_ddraw.h"
#include "gtk_domain.h"
#include "domains.h"


int contains_int_point( Polyhedron * );

int pgcd(int a, int b)
{
        int r;
 
        if( a== 0 )
                return( abs(b) );
 
        if( b==0 )
                return( abs(a) );
 
        do
        {
                r = a % b;
                a = b;
                b = r;
        }
        while ( r!=0 );
 
        return( abs(a) );
}
 


/* Parcours de polyedre */
/* Usage :
	mat = (Matrix *)Scanning_Matrix( Polyhedron *P, Matrix *order, int ws )
P is the Polyhedron to be scanned
ws the working space for the polylib calls
order a matrix containing the indices to be scanned (destroyed by this call) :
exemple :
	order = [ 0 1 0 0 0 ]
	        [ 0 0 1 0 0 ]
	        [ 0 0 0 1 0 ]
	to scan a polyhedron [i,j,k], with one parameter M

This call to Scanning_Matrix returns a matrix of the following form :
		[0    i     j   k   M  cte]
		0/1 *(!=0)  0   0   *   *
		           ...
		0/1  *!0    0   0   *   *
		0/1   *    *!0  0   *   *
		           ...
		0/1   *    *!0  0   *   *
		0/1   *     *  *!0  *   *
		           ...
		0/1   *     *  *!0  *   *
*/

static void scan_r( Polyhedron *P, Matrix *forall_ray, Matrix *C, int ws )
{
	int l, c;

	/* elimine les contraintes sur les termes restants
		en ajoutant des lines dans ces directions. */
	/* forall_ray =  0 1     c   d  d+1
	                 0 0..0  1 0..0   0
	                          ........
	*/

	for( c=1 ; c<forall_ray->NbColumns ; ++c )
		if( forall_ray->p[0][c] )
			break;
	/* elimine la premiere ligne de la matrice forall_ray */
	if( --forall_ray->NbRows )
	{	Polyhedron *Psimp, *DU;
		memcpy( &forall_ray->p[0][0], &forall_ray->p[1][0],
					sizeof(int)*forall_ray->NbColumns*forall_ray->NbRows );
		DU = DomainAddRays( P, forall_ray, ws );

		/* la borne min et max en fct des precedents est donnee par DU */
		/* ajoute la matrice de contraintes de DU a C pour les lignes de
			DU contenant un !=0 a l'indice c */

		for( l=0 ; l<DU->NbConstraints ; ++l )
		{
			if( DU->Constraint[l][c] )
			{
				memcpy( &C->p[C->NbRows][0], &DU->Constraint[l][0],
								sizeof(int)*(DU->Dimension+2) );
				++C->NbRows;
			}
		}

		/* simplifie P en donnant DU comme contexte */
		Psimp = DomainSimplify( P, DU, ws );
		Polyhedron_Free( DU );

		scan_r( Psimp, forall_ray, C, ws );

		Domain_Free( Psimp );
	}
	else
	{
		/* ajoute la matrice de contraintes restant (dans P) a C */
		for( l=0 ; l<P->NbConstraints ; ++l )
			memcpy( &C->p[l+C->NbRows][0], &P->Constraint[l][0],
					sizeof(int)*(P->Dimension+2) );
		C->NbRows += P->NbConstraints;
	}
}

/* passer en parametre l'ordre dans lequel on veut executer la boucle. */
/* sous forme de vecteur ( {1,2,3,...} si ordre i->j->...) */
Matrix *Scanning_Matrix( Polyhedron *P, Matrix *order, int ws )
{
	Matrix *MRes;

	if( order->NbColumns != P->Dimension+2 )
	{
		Warn("? Scanning_Matrix: operation on different dimensions\n");
		return NULL;
	}

/*	if( P->next )
	{
		Warn("? Scanning_Matrix: can't scan a union of domains\n");
		return NULL;
	}
*/


/*	Mtmp = Matrix_Alloc( P->Dimension, P->Dimension+2 );
 *	for( l=0 ; l<P->Dimension ; ++l )
 *	{
 *		for( i=0 ; i<Mtmp->NbColumns ; ++i )
 *			Mtmp->p[l][i] = 0;
 *		if( order->p[l]<1 || order->p[l]>P->Dimension )
 *			fprintf(stderr,"? Scanning_Matrix: incorrect order vector\n");
 *		Mtmp->p[l][order->p[l]] = 1;
 *	}
 */

	MRes = Matrix_Alloc( ws, P->Dimension+2 );
	MRes->NbRows = 0;

	scan_r( P, order, MRes, ws );

/*	Matrix_Free( Mtmp ); */

	if( MRes->NbRows == 0 )
	{
		Matrix_Free( MRes );
		return NULL;
	}
	else
	{	/* trop de lignes sont allouees...
			on recopie cette matrice dans une nouvelle */
		Matrix *M;
	 	M = Matrix_Alloc( MRes->NbRows, MRes->NbColumns );
		memcpy( &M->p[0][0], &MRes->p[0][0],
					sizeof(int)*MRes->NbRows*MRes->NbColumns );
		Matrix_Free( MRes );
		return( M );
	}
}



/* parcours d'un polyhedre convexe (pas union) avec appel de fonction */
/* pour chaque pt entier */
/* USAGE :	FORALL_P_do( Polyhedron *P, int f(int *) )
		la fonction f est appelee en passant le pt courant en parametre */
/* le parcours s'arrete si f renvoie FALSE */
/* la fonction renvoie FALSE si elle s'est arretee avant la fin */
/* c = indice parcourant 0 -> NbC-2 inclus */
static int FORALL_P_at_pt_do( Matrix *C, int c, int *pt, int f() )
{
	int l,i,j;
	int min_num=0, max_num=0, new_num;
	int min_den=1, max_den=1, new_den;
	int trouve_min, trouve_max;
	int g, den;

/*
Cherche une ligne du type :
         0   1 ... c+1  c+2 .. NbC-2 NbC-1
    C =  *   * .... *    0 ....  0     *
*/

	trouve_min = trouve_max = 0;

	/* on parcourt tous les !=0 sur cette colonne */
	for( l=0 ; l<C->NbRows ; ++l )
	{  
		if( C->p[l][c+1] == 0 )
			continue;

		/* si sur cette ligne,
				il y a un !=0 apres cet indice on ignore la ligne */
		for( j=c+2 ; j<C->NbColumns-1 ; ++j )
			if( C->p[l][j] )
				break;
		if( j<C->NbColumns-1 )
			continue;

		/* OK, la ligne 'l' contient bien une ref utile a cette variable */

		/* cherche la valeur de i : cte +val des ind. precedent */
		new_num = C->p[l][C->NbColumns-1];
		for( j=1 ; j<=c ; ++j )
			new_num += pt[j-1] * C->p[l][j];
		new_den = C->p[l][c+1];
		if( new_den < 0 )			/* denominateurs toujours > 0 */
			new_den = -new_den;
		else
			new_num = -new_num;
		if( C->p[l][0] )
		{
			/* inegalite */
			if( C->p[l][c+1] > 0 )
			{
				/* new est un nouveau min */
				if( trouve_min )
				{	if( new_num*min_den > min_num*new_den )
					{	min_num = new_num;
						min_den = new_den;
					}
				}
				else
				{	min_num = new_num;
					min_den = new_den;
					trouve_min = 1;
				}
			}
			else
			{	/* new est un nouveau max */
				if( trouve_max )
				{	if( new_num*max_den < max_num*new_den )
					{	max_num = new_num;
						max_den = new_den;
					}
				}
				else
				{	max_num = new_num;
					max_den = new_den;
					trouve_max = 1;
				}
			}
			/* si le min est devenu > au max pas la peine de continuer */
			if( trouve_min && trouve_max )
				if( max_num*min_den < min_num*max_den )
					return 1;
		}
		else
		{	/* egalite */
			/* si on a deja un min ou un max qui empiete on s'arrete la. */
			if( trouve_min && (new_num*min_den < min_num*new_den) )
				return 1;
			if( trouve_max && (new_num*max_den > max_num*new_den) )
				return 1;
			trouve_min = 1;
			trouve_max = 1;
			min_num = max_num = new_num;
			min_den = max_den = new_den;
		}
	}

	if( !trouve_min || !trouve_max )
	{
		fprintf( stderr, "Error in scan polyhedron : non closed polyhedron\n" );
		Matrix_Print( stderr, "%3d ", C );

		Warn( "Can't visualize an infinite polyhedron\n" );

		return 0;
	}


	g = pgcd( min_den, max_den );
	den = min_den * max_den / g;
	min_num = min_num * max_den/g;
	max_num = max_num * min_den/g;

	/* se place sur le premier "vrai" */
	if( min_num < 0 )
		min_num += (-min_num)%den;
	else
		if( (min_num)%den )
			min_num += den - (min_num)%den;

	/* decrit la boucle */
	if( c == C->NbColumns-3 )
	{	for( i=min_num ; i<=max_num ; i+=den )
		{
			pt[c] = i/den;
			if( !f(pt) )
				return 0;
		}
	}
	else
	{	for( i=min_num ; i<=max_num ; i+=den )
		{
			pt[c] = i/den;
			if( !FORALL_P_at_pt_do( C, c+1, pt, f ) )
				return 0;
		}
	}
	return 1;
}

int FORALL_P_do( Polyhedron *P, int *pt, int f() )
{
	Matrix *constraints;
	Matrix *o;
	int i;

	if( !P || P->NbRays==0 )
		return 1;

	o = Matrix_Alloc( P->Dimension, P->Dimension+2 );
	memset( &o->p[0][0], 0, o->NbRows*o->NbColumns*sizeof(int) );
	for( i=0 ; i<o->NbRows ; ++i )
		o->p[i][i+1] = 1;

	constraints = Scanning_Matrix( P, o, WS );
	Matrix_Free( o );

	if( constraints )
		return( FORALL_P_at_pt_do( constraints, 0, pt, f ) );
	else
		return 1;


}

#ifdef lkmjnhbrsdvfu
static Display *_ddD;
static Window _ddW;
static Equation *_duEq;
static Occurrence *_duOcc;
static Matrix *_dudz0ray;
static int DrawUtil( int *pt )
{
	Polyhedron *U;

	U = DUtil( _duEq->DNP, _duOcc->RNP, _dudz0ray );

	if( U && U->NbRays>0 )
	{
		if( U->Dimension==DimNP )
		{	if( contains_int_point(U) )
				Draw_One_Util( _ddD,_ddW, pt, U,
					_duOcc->U, &_duOcc->visu, _duOcc->T->donnee );
		}
		else
		{
			printf( "Internal error. Wrong dimension util. set\n");
			return 0;
		}
	}
	return 1;
}
void Draw_Dep( Display *display,Window xwin )
{
	Terme *t;
	Equation *e;
	Occurrence *o;

	_ddD = display;
	_ddW = xwin;
	for( t=Ttab ; t ; t=t->next )
	 for( e=t->Eq ; e ; e=e->next )
	 {
		if( !e->DNP )
			continue;
	 	_duEq = e;
		_dudz0ray = Matrix_Alloc( 1, DimNP+2 );
		_dudz0ray->p[0][0] = 1;
		_dudz0ray->p[0][e->DNP->Dimension+1] = 1;
		for( o=e->Occ ; o ; o=o->next )
		{
			if( o->visu.visu )
			{
				if( e->DNP->Dimension < DimNP )
				{	/* vers une dimension inferieure: on ne dessine que emit. */
					Draw_Int_Points( display,xwin, o->Emit.DNP, COL_EMIT );
				}
				else
				{
					if( o->visu.One  )
					{
						if( o->visu.un_Util )
							Draw_One_Util( display,xwin, o->visu.Show_pt_Util,
								o->visu.un_Util, o->U, &o->visu, o->T->donnee );
					}
					else
					{
						_duOcc = o;
						_dudz0ray->p[0][o->Emit.DNP->Dimension+1] = 1;
						_dudz0ray->NbColumns = o->Emit.DNP->Dimension+2;
						FORALL_P_do( o->Emit.DNP, &_dudz0ray->p[0][1], DrawUtil );
					}
				}
			}
		}
		Matrix_Free( _dudz0ray );
	 }
}

#endif

static int _CIP_ok( int *pt )
{	return 0;
}
int contains_int_point( Polyhedron *P )
{	int r, *pt;
	pt = malloc( sizeof(int)*P->Dimension );
	r = FORALL_P_do(P, pt, _CIP_ok );
	free( pt );
	return( !r );
}

static int compt;
static int compte_nb_pts_poly( int *pt )
{
	++compt;
	return 1;
}
/* compte le nb de pts entiers dans un polyedre */
int nb_pts_poly( Polyhedron *P )
{
	int *pt;
	pt = malloc( sizeof(int)*P->Dimension );
	compt = 0;
 	FORALL_P_do(P, pt, compte_nb_pts_poly );
	free( pt );

	return( compt );
}

/* parallelisme potentiel */
int PP()
{
/*int pp, i;
	Polyhedron *P, *n;

	if( !validFile )
		return( 0 );
	pp = 0;
	for( P=Timing_surf ; P ; )
	{
		n = P->next;
		P->next = NULL;
		i = nb_pts_poly( P );
		if( i > pp )
			pp = i;
		P->next = n;
		P = n;
	}
	return( pp );
*/
return 0;
}


/* teste si le point de coord. pt est dans P */
int DansPolyedre( Polyhedron *P, int *pt )
{
	int c, i, r;

	for( c=0 ; c<P->NbConstraints ; ++c )
	{
		r = 0;
		for( i=0 ; i<P->Dimension ; ++i )
		{
			r += pt[i] * P->Constraint[c][i+1];
		}
		r += P->Constraint[c][P->Dimension+1];	/* constante */

		if( P->Constraint[c][0] )	/* inegalite ou egalite */
		{
			if( r < 0 )
				return( 0 );
		}
		else
		{
			if( r != 0 )
				return( 0 );
		}
	}
	return( 1 );
}

